﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using WRECWrapper;

namespace WRECClient
{
    class Program
    {
        static void Main(string[] args)
        {
            // Get token from config file.
            var token = ConfigurationManager.AppSettings["token"];
            // Overwrite token for debugging:
            token = "YOUR-TOKEN-HERE";

            var connectorURL = ConfigurationManager.AppSettings["connector_url"];
            var wrecActivationService = new ActivateService(token, connectorURL);
            var wrecOutputService = new OutputService(token, connectorURL);

            try
            {
                processContracts(wrecActivationService);
                processRealEstateProperties(wrecActivationService, wrecOutputService);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine("Process aborted.");
            }
            Console.WriteLine("Press the 'any key'.");
            Console.ReadKey();
        }

        private static void processRealEstateProperties(ActivateService wrecActivationService, OutputService wrecOutputService)
        {
            var mediaContracts = getAllContracts(wrecActivationService);

            // Process pending activation requests. ONLY applicable in case 'Auto Accept = false'.
            var activeMediaContract = mediaContracts.FirstOrDefault(m => m.MediaContractStatus.Equals("ACTIVE", StringComparison.InvariantCultureIgnoreCase));
            
            // Get the list of all real estate property summaries (example).
            var realEstatePropertysummaries = getRealEstatePropertySummaries(wrecOutputService, activeMediaContract);

            // Get the property details (example).
            var realEstateProperty = getRealEstateProperty(wrecOutputService, realEstatePropertysummaries);

            // Confirm to the WREC (and the Realtor) that the Real Estate Property was succesfully processed.
            // UNCOMMENT TO EXECUTE.
            // confirmRealEstatePropertyRetrieval(wrecOutputService, realEstatePropertysummaries);

            // Inform the WREC (and the Realtor) that the Real Estate Property was NOT succesfully processed.
            // UNCOMMENT TO EXECUTE.
            //rejectRealEstatePropertyRetrieval(wrecOutputService, realEstatePropertysummaries);     
        }

        private static RealEstateProperty getRealEstateProperty(OutputService wrecOutputService, IList<RealEstatePropertySummarySnapshot> realEstatePropertysummaries)
        {
            if ((realEstatePropertysummaries == null) || (realEstatePropertysummaries.Count == 0))
                return null;

            return wrecOutputService.GetRealEstateProperty(realEstatePropertysummaries[0].RealtorID, realEstatePropertysummaries[0].RealEstateProperyID);
        }

        private static IList<RealEstatePropertySummarySnapshot> getRealEstatePropertySummaries(OutputService wrecOutputService, MediaContractSnapshot mediaContractSnapshot)
        {
            if (mediaContractSnapshot == null)
                return new List<RealEstatePropertySummarySnapshot>();

            var summaries = wrecOutputService.GetRealEstatePropertySummarySnapshots(mediaContractSnapshot.RealtorID);
            Console.WriteLine(string.Format("There are '{0}' real estate property summaries found for realtor '{1}'.", summaries.Count, mediaContractSnapshot.RealtorID));
            if (summaries.Count > 0)
            {
                foreach (var summary in summaries)
                {
                    Console.WriteLine(string.Format("Real estate property '{0}' with status '{1}' found for Realtor '{2}'.",
                                                    summary.AddressSummary, summary.RealEstateProperyStatus, mediaContractSnapshot.RealtorID));
                }
            }
            return summaries;
        }
        private static void confirmRealEstatePropertyRetrieval(OutputService wrecOutputService,
                                                               IList<RealEstatePropertySummarySnapshot> realEstatePropertysummaries)
        {
            if ((realEstatePropertysummaries != null) && (realEstatePropertysummaries.Count > 0))
            {
                // Example: confirm the real estate property retrieval of the first real estate property.
                var realEstateProperty = realEstatePropertysummaries[0];
                wrecOutputService.ConfirmRealEstatePropertyRetrieval(realEstateProperty.RealtorID,
                                                                     realEstateProperty.RealEstateProperyID,
                                                                     "http://www.myportal.com/details.aspx?objectID=112233",
                                                                     "Het pand is bijgewerkt op onze portal.");
            }
        }
        private static void rejectRealEstatePropertyRetrieval(OutputService wrecOutputService, IList<RealEstatePropertySummarySnapshot> realEstatePropertysummaries)
        {
            if (realEstatePropertysummaries.Count > 1)
            {
                // Example: reject the real estate property retrieval of the second real estate property.
                var realEstateProperty = realEstatePropertysummaries[1];
                wrecOutputService.RejectRealEstatePropertyRetrieval(realEstateProperty.RealtorID,
                                                                    realEstateProperty.RealEstateProperyID,
                                                                    "Het pand heeft geen foto's. Zonder foto's kan het pand niet op onze portal geplaatst worden.");
            }
        }

        private static void processContracts(ActivateService wrecActivationService)
        {
            // Process pending activation requests. ONLY applicable in case 'Auto Accept = false'.
            // UNCOMMENT TO EXECUTE.
            // processPendingActivationRequests(wrecActivationService);

            // Process pending de-activation requests. ONLY applicable in case 'Auto Accept = false'.
            // UNCOMMENT TO EXECUTE.
            // processPendingDeactivationRequests(wrecActivationService);

            var mediaContracts = getAllContracts(wrecActivationService);

            // Suspend a media contract (e.g. when the Realtor who owns the Contract doesn't pay the bills).
            // UNCOMMENT TO EXECUTE.
            // var suspendedMediaContract = suspendMediaContract(wrecActivationService, mediaContracts);
            // Re-enable the suspended media contract.
            // UNCOMMENT TO EXECUTE.
            // var activeMediaContract = reactivateMediaContract(wrecActivationService, suspendedMediaContract);
        }

        private static void processPendingActivationRequests(ActivateService activateService)
        {
            // Get a list of pending contracts.
            var pendingActiveContracts = activateService.GetMediaContractSnapshots(ActivateService.ContractState.PENDING_ACTIVATION);
            Console.WriteLine(string.Format("There are '{0}' pending activation requests for contracts.", pendingActiveContracts.Count));
            if (pendingActiveContracts.Count > 0)
            {
                foreach (var pendingActiveContract in pendingActiveContracts)
                {
                    Console.WriteLine(string.Format("Pending activation request for a contract '{0}' found for Realtor '{1}'.",
                                                    pendingActiveContract.MediaContractID, pendingActiveContract.Name));
                    Console.WriteLine();

                    if (pendingActiveContract.RealtorID == 344)
                    {
                        // Accept pending contract request. 
                        // UNCOMMENT TO EXECUTE.
                        // activateService.AcceptActivation(pendingActiveContract.MediaContractID);
                    }
                    // Reject pending contract request.
                    // UNCOMMENT TO EXECUTE.
                    //activateService.RejectActivationRequest(pendingActiveContract.MediaContractID, "Contract not returned.");
                }
            }
        }
        private static void processPendingDeactivationRequests(ActivateService activateService)
        {
            // Get a list of all pending de-activation for contract requests.
            var pendingDeactiveContracts = activateService.GetMediaContractSnapshots(ActivateService.ContractState.PENDING_DEACTIVATION);
            Console.WriteLine(string.Format("There are '{0}' pending de-activation for contract requests.", pendingDeactiveContracts.Count));
            if (pendingDeactiveContracts.Count > 0)
            {
                foreach (var pendingDeactiveContract in pendingDeactiveContracts)
                {
                    Console.WriteLine(string.Format("Pending de-activation request for a contract '{0}' found for Realtor '{1}'.",
                                                    pendingDeactiveContract.MediaContractID, pendingDeactiveContract.Name));
                    Console.WriteLine();

                    // Accept pending contract request.
                    activateService.AcceptDeactivation(pendingDeactiveContract.MediaContractID);
                }
            }
        }
        private static IList<MediaContractSnapshot> getAllContracts(ActivateService activateService)
        {
            // Get a list of all contracts.
            var mediaContracts = activateService.GetMediaContractSnapshots();
            Console.WriteLine(string.Format("There are {0} contracts.", mediaContracts.Count));
            if (mediaContracts.Count > 0)
            {
                foreach (var contract in mediaContracts)
                {
                    Console.WriteLine(string.Format("Contract '{0}' found for Realtor '{1}'. Status: '{2}'", contract.MediaContractID,
                                                    contract.Name, contract.MediaContractStatus));
                    Console.WriteLine();

                    // Suspend a contract.
                    if (contract.MediaContractStatus.Equals("active", StringComparison.InvariantCultureIgnoreCase))
                    {
                        // UNCOMMENT TO EXECUTE.
                        // activateService.SuspendContract(contract.MediaContractID, "Please pay bill 202120202.");
                    }

                    // Reactivate a contract.
                    if (contract.MediaContractStatus.Equals("suspended", StringComparison.InvariantCultureIgnoreCase))
                    {
                        // UNCOMMENT TO EXECUTE.
                        // activateService.ReactivateContract(contract.MediaContractID);
                    }
                }
            }
            return mediaContracts;
        }

        private static MediaContractSnapshot suspendMediaContract(ActivateService activateService, IList<MediaContractSnapshot> mediaContracts)
        {
            if (mediaContracts == null)
                return null;

            var activeMediaContract = mediaContracts.FirstOrDefault(m => m.MediaContractStatus.Equals("ACTIVE", StringComparison.InvariantCultureIgnoreCase));
            if (activeMediaContract != null)
            {
                activateService.SuspendContract(activeMediaContract.MediaContractID, "Neem s.v.p. contact op met onze debiteurenadministratie op telefoonnummer 012-3456789.");
            }
            return activeMediaContract;
        }
        private static MediaContractSnapshot reactivateMediaContract(ActivateService activateService, MediaContractSnapshot suspendedMediaContract)
        {
            if (suspendedMediaContract == null)
                return null;
            activateService.ReactivateContract(suspendedMediaContract.MediaContractID);
            return suspendedMediaContract;
        }
    }
}
